var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as auth from '../auth';
import { server } from '../__mocks__/server';
import { rest } from 'msw';
import querystring from 'querystring';
import { mockDate } from '../__mocks__/mockDate';
import { getLoggedUserData, updateUserData } from '../auth';
const testData = {
    ADMIN_USER: 'test@test.pl',
    ADMIN_PASS: 'hello',
    TENANT: 'tenant123',
    VENUE: 'venue1',
    BASE_URL: 'http://localhost',
    BASIC_AUTH: 'abc',
};
test('getTokenWithPassword', () => __awaiter(void 0, void 0, void 0, function* () {
    const authSuccess = yield auth.authorizeWithUserPass(testData.BASE_URL, testData.TENANT, testData.BASIC_AUTH, testData.ADMIN_USER, testData.ADMIN_PASS);
    expect(authSuccess).toBe(true);
}));
test('getTokenWithRefreshToken', () => __awaiter(void 0, void 0, void 0, function* () {
    const authSuccess = yield auth.authorizeWithRefreshToken(testData.BASE_URL, testData.TENANT, testData.BASIC_AUTH, 'proper_refresh_token');
    expect(authSuccess).toBe(true);
}));
test('getAuthData - makes only 1 API call with multiple concurrent request for access_token having refresh_token', () => __awaiter(void 0, void 0, void 0, function* () {
    const dispatchRequest = jest.fn();
    server.events.on('request:start', dispatchRequest);
    const tokenProvider = {
        getRefreshToken: jest.fn().mockResolvedValue('proper_refresh_token'),
        setRefreshToken: jest.fn().mockResolvedValue(undefined),
        clearRefreshToken: jest.fn().mockResolvedValue(undefined),
    };
    // Clear _authData from other tests
    auth.clearAuthData('any_tenant', tokenProvider);
    const results = yield Promise.all(Array(10)
        .fill(0)
        .map(() => auth.authDataProvider(testData, tokenProvider)));
    expect(results.every((result) => result.token === 'proper_access_token')).toBe(true);
    expect(dispatchRequest).toHaveBeenCalledTimes(1);
}));
test('getAuthData - makes 10 API calls with multiple concurrent request if network error', () => __awaiter(void 0, void 0, void 0, function* () {
    server.use(rest.post(authUrl, (req, resp, ctx) => {
        return resp.networkError('Failed to connect');
    }));
    const dispatchRequest = jest.fn();
    server.events.on('request:start', dispatchRequest);
    const tokenProvider = {
        getRefreshToken: jest.fn().mockResolvedValue('proper_refresh_token'),
        setRefreshToken: jest.fn().mockResolvedValue(undefined),
        clearRefreshToken: jest.fn().mockResolvedValue(undefined),
    };
    // Clear _authData from other tests
    auth.clearAuthData('any_tenant', tokenProvider);
    const results = yield Promise.all(Array(10)
        .fill(0)
        .map(() => auth.authDataProvider(testData, tokenProvider)));
    expect(results.every((result) => result.token === '')).toBe(true);
    expect(dispatchRequest).toHaveBeenCalledTimes(10);
}));
test('getTokenWithRefreshToken - wrong token', () => __awaiter(void 0, void 0, void 0, function* () {
    const authSuccess = yield auth.authorizeWithRefreshToken(testData.BASE_URL, testData.TENANT, testData.BASIC_AUTH, 'invalid_token');
    expect(authSuccess).toBe(false);
}));
const authUrl = `${testData.BASE_URL}/auth-oauth2/oauth/token`;
test('getAuthData - return existing token if it is still valid ', () => __awaiter(void 0, void 0, void 0, function* () {
    const refreshHandler = {
        getRefreshToken: (tenant) => new Promise((resolve) => resolve('TEST_REFRESH_TOKEN')),
        setRefreshToken: (tenant) => new Promise((resolve) => resolve()),
        clearRefreshToken: (tenant) => new Promise((resolve) => resolve()),
    };
    const ctx = {
        BASE_URL: testData.BASE_URL,
        TENANT: testData.TENANT,
        BASIC_AUTH: testData.BASIC_AUTH,
        anonymousAuth: true,
    };
    const timeFreeze = new Date('2020-10-01T00:10:01.30Z'); //zero time
    const resetDateMock = mockDate(timeFreeze);
    auth.setAuthData('tenant1', {
        expires_in: '3600',
        access_token: '123456',
        UUID: 'user1',
        refresh_token: 'refresh1',
    }, refreshHandler, new Date('2020-10-01T00:09:01.30Z').valueOf());
    const { token, UUID } = yield auth.authDataProvider(ctx, refreshHandler, false);
    expect(token).toBe('123456');
    resetDateMock();
}));
test('getAuthData - use refresh token to retrieve new token if existing token expired ', () => __awaiter(void 0, void 0, void 0, function* () {
    server.use(rest.post(authUrl, (req, resp, ctx) => {
        const params = querystring.parse('' + req.body);
        const grantType = params['grant_type'];
        if (grantType === 'refresh_token') {
            const refreshToken = params['refresh_token'];
            if (refreshToken === 'TEST_REFRESH_TOKEN') {
                return resp(ctx.status(200), ctx.json({
                    access_token: '123456',
                    expires_in: '3600',
                    UUID: 'user_uuid123',
                    refresh_token: 'proper_refresh_token',
                }));
            }
        }
        return resp(ctx.status(403));
    }));
    const refreshHandler = {
        getRefreshToken: (tenant) => 'TEST_REFRESH_TOKEN',
        setRefreshToken: (tenant) => { },
        clearRefreshToken: (tenant) => { },
    };
    const ctx = {
        BASE_URL: testData.BASE_URL,
        TENANT: testData.TENANT,
        BASIC_AUTH: testData.BASIC_AUTH,
        anonymousAuth: true,
    };
    const timeFreeze = new Date('2020-10-01T12:10:01.30Z'); //zero time
    const resetDateMock = mockDate(timeFreeze);
    auth.setAuthData('tenant1', {
        expires_in: '3600',
        access_token: '123456',
        UUID: 'user1',
        refresh_token: 'refresh1',
    }, refreshHandler, new Date('2020-10-01T11:09:01.30Z').valueOf());
    const { token, UUID } = yield auth.authDataProvider(ctx, refreshHandler, false);
    expect(token).toBe('123456');
    expect(UUID).toBe('user_uuid123');
    resetDateMock();
}));
const userDataUrl = `${testData.BASE_URL}/auth-api/api/me`;
const validUserData = {
    firstName: 'testname',
    email: 'email@email.com',
    UUID: 'uuid_1',
};
beforeEach(() => {
    // Setup server
    server.use(rest.get(userDataUrl, (req, res, ctx) => {
        if (req.headers.get('authorization') === 'Bearer VALID_TEST_TOKEN') {
            return res(ctx.status(200), ctx.json(validUserData));
        }
    }));
    server.use(rest.post(userDataUrl, (req, res, ctx) => {
        if (req.headers.get('authorization') === 'Bearer VALID_TEST_TOKEN') {
            return res(ctx.status(200), ctx.json(true));
        }
        return res(ctx.status(401));
    }));
});
test('getLoggedUserData', () => __awaiter(void 0, void 0, void 0, function* () {
    const userData = yield getLoggedUserData(testData.BASE_URL, 'VALID_TEST_TOKEN');
    expect(userData).toMatchObject(validUserData);
}));
test('updateUserData', () => __awaiter(void 0, void 0, void 0, function* () {
    const result = yield updateUserData(testData.BASE_URL, 'VALID_TEST_TOKEN', {
        phone: '123',
        firstName: 'testname',
    });
    expect(result).toBe(true);
}));
test('updateUserData - invalid token', () => __awaiter(void 0, void 0, void 0, function* () {
    const onError = jest.fn();
    yield updateUserData(testData.BASE_URL, 'INVALID_TEST_TOKEN', {
        phone: '123',
        firstName: 'testname',
    }).catch(onError);
    expect(onError).toHaveBeenCalled();
}));
